dynamic-cabal
=============

[![Build Status](https://secure.travis-ci.org/bennofs/dynamic-cabal.png?branch=master)](http://travis-ci.org/bennofs/dynamic-cabal)

If you've ever used Cabal together with the GHC-API, you know the problem. Because GHC depends on a version of Cabal, which is often outdated, there is no way to parse the setup-config file generated by newer cabal versions. This library attemps to solve the problem by dynamically generating code that performs the action you want, and then compiling and loading that with GHC. With this method, you don't need to depend on Cabal at compile time and so you can use any version of Cabal.

## Usage

Currently, the library only allows two queries: Getting the targets (along with their dependencies, ghc options, etc) and the package databases. The first is easily achieved using the `targets` query provided by the library. To run the query, you can use the `runQuery` function, which takes the path to the setup-config file as an argument. For example, the following little program prints out the names of all test suites, when run in a configured cabal project root directory:

```haskell
import Distribution.Client.Dynamic

main :: IO ()
main = do
  tgs <- runQuery (on localPkgDesc targets) "dist/setup-config"
  mapM_ putStrLn [ n | TestSuite n <- map name tgs ]
```

Because `targets` works on a PackageDescription, `on localPkgDesc` is used to get the current PackageDescription.

## Contributing

At the moment, I only implemented the functions I need myself. If you have more functions you want to implement, just send a pull request or open an issue.
